<?php

defined('ABSPATH') || die('error');

/**
 * Entry point for building the backend settings page
 */
function gdprhelper__main_backend($pluginfile): void
{
    require_once(GH__PLUGIN_INCLUDES_PATH . '/includes/magicStrings.php');

    add_action('admin_init', 'gdprhelper__register_settings');
    add_action('admin_menu', 'gdprhelper__register_admin_menu');
    add_action('admin_enqueue_scripts', 'gdprhelper__enqueue_scripts');
    register_activation_hook($pluginfile, 'gdprhelper__first_time_init');
}

/**
 * Called for the very first start of this plugin. Will populate the DB option with a default value
 */
function gdprhelper__first_time_init(): void
{
    $content_maps = __('Show map?', GH__TEXT_DOMAIN);
    $content_maps .= "\r\n\r\n";
    $content_maps .= __('There is a map from Google Map or OpenStreetMap embedded here. Do you want to show this map? Doing so will lead to your personal data (your IP address) being sent to Google or OpenStreetMap, which can be used to track you.', GH__TEXT_DOMAIN);
    $content_maps .= "\r\n\r\n";
    $content_maps .= __('Use the button below to enable this map. A cookie will be stored on you computer to remeber your descision (no personal data will be stored).', GH__TEXT_DOMAIN);

    $default = [
        GH__SETTINGS_FIELDS__ENABLED => false,
        GH__SETTINGS_FIELDS__ENABLED_WHEN_LOGGED_IN,
        GH__SETTINGS_FIELDS__REPORT_ONLY => false,
        GH__SETTINGS_FIELDS__REPORT_URI => '',
        GH__SETTINGS_FIELDS__CSP_STRING => "default-src 'self' 'unsafe-inline' data:",
        GH__SETTINGS_FIELDS__MAPS_HELPER_ENABLED => false,
        GH__SETTINGS_FIELDS__MAPS_PACEHOLDER_IMAGE => '',
        GH__SETTINGS_FIELDS__PRIVACY_URL => '',
        GH__SETTINGS_FIELDS__CUSTOM_PRIVACY_TEXT => $content_maps
    ];
    update_option(GH__SETTINGS_PAGE__ID, $default);
}

/**
 * Enqueue admin scripts and stylesheets
 * 
 * @param string $hook Name of the current (admin) page
 */
function gdprhelper__enqueue_scripts(string $hook): void
{
    if ($hook != 'settings_page_' . GH__SETTINGS_PAGE__ID) {
        return;
    }

    wp_enqueue_media();

    wp_enqueue_script('gh__backend_js', GH__PLUGIN_URL . '/js/backend_script.js', array('jquery'), GH__CURRENT_VERSION, true);
    wp_enqueue_style('gh__backend_css', GH__PLUGIN_URL . '/css/backend_style.css', [], GH__CURRENT_VERSION);
}

/**
 * Registers the settings inside WP
 */
function gdprhelper__register_settings(): void
{
    register_setting(GH__SETTINGS_PAGE__ID, GH__SETTINGS_PAGE__ID);

    add_filter('pre_update_option_' . GH__SETTINGS_PAGE__ID, 'gdprhelper__pre_update_filter');

    add_settings_section(
        'gh_setting_section_name',
        __('CSP Settings for GDPR', GH__TEXT_DOMAIN),
        function () use ($description) {
            echo esc_html($description);
        },
        GH__SETTINGS_PAGE__ID
    );

    add_settings_field(
        GH__SETTINGS_FIELDS__ENABLED,
        __('Enabled', GH__TEXT_DOMAIN),
        'gdprhelper__renderer__checkbox_input',
        GH__SETTINGS_PAGE__ID,
        GH__SETTINGS_PAGE__SECTION_NAME,
        array(
            GH__SETTINGS_FIELDS__ENABLED,
            __('Only when enabled will the CSP-header be set.', GH__TEXT_DOMAIN)
        )
    );

    add_settings_field(
        GH__SETTINGS_FIELDS__ENABLED_WHEN_LOGGED_IN,
        __('Enabled when logged in', GH__TEXT_DOMAIN),
        'gdprhelper__renderer__checkbox_input',
        GH__SETTINGS_PAGE__ID,
        GH__SETTINGS_PAGE__SECTION_NAME,
        array(
            GH__SETTINGS_FIELDS__ENABLED_WHEN_LOGGED_IN,
            __('When enabled the CSP will also be applied when logged in and on the admin backend. Not recommended.', GH__TEXT_DOMAIN)
        )
    );

    add_settings_field(
        GH__SETTINGS_FIELDS__REPORT_ONLY,
        __('Report only', GH__TEXT_DOMAIN),
        'gdprhelper__renderer__checkbox_input',
        GH__SETTINGS_PAGE__ID,
        GH__SETTINGS_PAGE__SECTION_NAME,
        array(
            GH__SETTINGS_FIELDS__REPORT_ONLY,
            __('Check if you want the CSP to only report. It will effect your site in any way in report mode. For this to work you must also provide a report uri below.', GH__TEXT_DOMAIN)
        )
    );

    add_settings_field(
        GH__SETTINGS_FIELDS__REPORT_URI,
        __('Report Uri', GH__TEXT_DOMAIN),
        'gdprhelper__renderer__text_input',
        GH__SETTINGS_PAGE__ID,
        GH__SETTINGS_PAGE__SECTION_NAME,
        array(
            GH__SETTINGS_FIELDS__REPORT_URI,
            __('If set to report only then please provide an URI to where the reports should be sent.', GH__TEXT_DOMAIN)
        )
    );

    add_settings_field(
        GH__SETTINGS_FIELDS__CSP_STRING,
        __('Content-Security-Policy', GH__TEXT_DOMAIN),
        'gdprhelper__renderer__textarea_input',
        GH__SETTINGS_PAGE__ID,
        GH__SETTINGS_PAGE__SECTION_NAME,
        array(
            GH__SETTINGS_FIELDS__CSP_STRING,
            __('This is the Content-Security-Policy that will be set in the http header.', GH__TEXT_DOMAIN)
        )
    );

    /* ############ Maps integration ############ */


    add_settings_field(
        GH__SETTINGS_FIELDS__MAPS_HELPER_ENABLED,
        __('Google Maps or OpenStreetMap enabled', GH__TEXT_DOMAIN),
        'gdprhelper__renderer__checkbox_input',
        GH__SETTINGS_PAGE__ID,
        GH__SETTINGS_PAGE__SECTION_NAME,
        array(
            GH__SETTINGS_FIELDS__MAPS_HELPER_ENABLED,
            sprintf(__('Only applicable if you use Elementor with the default Google Maps block or with the map block from the external %s plugin. If enabled the Maps/OpenStreetMap block will be overriden with a placeholder that asks visitors to accept the privacy policy. Upon doing so the map will be loaded and shown.', GH__TEXT_DOMAIN), '<a href="https://de.wordpress.org/plugins/osm-map-elementor/" target="_blank">OpenStreetMap</a>')
        )
    );

    add_settings_field(
        GH__SETTINGS_FIELDS__MAPS_PACEHOLDER_IMAGE,
        __('Placeholder image for Maps', GH__TEXT_DOMAIN),
        'gdprhelper__renderer__image_input',
        GH__SETTINGS_PAGE__ID,
        GH__SETTINGS_PAGE__SECTION_NAME,
        array(
            GH__SETTINGS_FIELDS__MAPS_PACEHOLDER_IMAGE,
            __('Select the placeholder image to show.', GH__TEXT_DOMAIN)
        )
    );

    add_settings_field(
        GH__SETTINGS_FIELDS__PRIVACY_URL,
        __('URL for the privacy policy', GH__TEXT_DOMAIN),
        'gdprhelper__renderer__text_input',
        GH__SETTINGS_PAGE__ID,
        GH__SETTINGS_PAGE__SECTION_NAME,
        array(
            GH__SETTINGS_FIELDS__PRIVACY_URL,
            __('The URL to your privacy policy.', GH__TEXT_DOMAIN)
        )
    );

    add_settings_field(
        GH__SETTINGS_FIELDS__CUSTOM_PRIVACY_TEXT,
        __('Custom Map placeholder text', GH__TEXT_DOMAIN),
        'gdprhelper__renderer__textarea_input',
        GH__SETTINGS_PAGE__ID,
        GH__SETTINGS_PAGE__SECTION_NAME,
        array(
            GH__SETTINGS_FIELDS__CUSTOM_PRIVACY_TEXT,
            __('This text will be shown in the placeholder.', GH__TEXT_DOMAIN)
        )
    );
}

function gdprhelper__pre_update_filter(array $new_value)
{
    foreach ($new_value as $key => $value) {
        $sanitized_value = $value;

        if ($key == GH__SETTINGS_FIELDS__CSP_STRING) {
            // Sanitize directive value, see https://www.w3.org/TR/CSP3/#framework-directives
            $regex = '/[^\x21-\x2B\x2D-\x3A\x3C-\x7E\x09\x20\n;]/u';
            $sanitized_value = preg_replace($regex, '', $value);
            // The newlines will be stripped before setting the http header. Leaving the in here will make it more readable for the user inside the texarea
        }
        $new_value[$key] = $sanitized_value;
    }

    return $new_value;
}

function gdprhelper__register_admin_menu(): void
{
    add_options_page(
        __('GDPR Helper', GH__TEXT_DOMAIN),
        __('GDPR Helper', GH__TEXT_DOMAIN),
        'manage_options',
        GH__SETTINGS_PAGE__ID,
        function () {
?>
        <div class="gh__backend-container">
            <h2><?php _e('Content Security Policy Manager', GH__TEXT_DOMAIN); ?></h2>
            <div>
                <p>
                <h3><?php _e('Please read this before using the plugin:', GH__TEXT_DOMAIN); ?></h3>
                <br />
                <?php
                echo wp_kses_post(__('This plugin should be used as an easy and fast way to block all outgoing request to other domains and webservers. The GDPR states that all usages of third party services must either be requiered to use this website/service or be explicitly accepted by the visitor. Many websites use incasive cookie banners that prevent visitors from using the website unless they comply with an extensive usage of third party services. We think that this makes the web inaccessable and cumbersome to use. Therefore we recommend more lightwheigt and out-of-the-way cookie banners as well as other tools that make sure the website functions as intended without using external services.', GH__TEXT_DOMAIN));
                ?>
                <br />
                <?php
                echo wp_kses_post(__('This plugin is only part of this solution. It will try to block all outgoing requests unless you explicitly allow them. We recommend using other plugins, like <i>OMGF</i> or <i>Extra privacy for Elementor</i>, to host external resources locally or at least ask the user for explicitly granting the access to third party services.', GH__TEXT_DOMAIN));
                ?>
                <br />
                <br />
                </p>

                <h3><?php _e('How this plugin should be used', GH__TEXT_DOMAIN); ?></h3>
                <p>
                    <?php
                    echo wp_kses_post(__('This plugin allows you to the a Content-Security-Policy for you website (more information <a href="https://developer.mozilla.org/en-US/docs/Web/HTTP/CSP" target="_blank">here</a> and <a href="https://content-security-policy.com/" target="_blank">here</a>).', GH__TEXT_DOMAIN));
                    ?>
                    <br />
                    <?php
                    echo wp_kses_post(__('
                    We recommend the following steps before acctivating this plugin:
                    <ol>
                        <li>
                            Host all your Google Fonts locally. We recommend using a plugin, like <a href="https://wordpress.org/plugins/host-webfonts-local/" target="_blank"><i>OMGF</i></a>
                        </li>
                        <li>
                            Do you use Google Maps? Use a plugin that allows you to hide it behind a dialog that gathers the vititors consent. For <i>Elementor</i> we recommend <a href="https://wordpress.org/plugins/extra-privacy-for-elementor/" target="_blank"><i>Extra Privacy for Elementor</i></a>
                        </li>
                        <li>
                            Do you use Google Recaptcha to secure a form? This will definetly load Google Fonts, which is not allowed without gathering consent first. We recomment switching to another captcha provider, like <a href="https://friendlycaptcha.com/" target="_blank">FriendlyCaptcha</a>. This one can be used with <a href="https://wordpress.org/plugins/wpforms-lite/" target="_blank"><i>WPForms</i></a>
                        </li>
                    </ol>', GH__TEXT_DOMAIN));
                    ?>
                    <br />
                    <?php
                    esc_html_e('Please note that loading external resources may lead to them loading Google Fonts. Therefore if you use external Services, like the above mentioned, please also gather consent for Google Fonts and include the neccessary text in your privacy policy!', GH__TEXT_DOMAIN);
                    ?>
                </p>
            </div>
            <form id="<?php echo GH__SETTINGS_PAGE__ID; ?>" action="options.php" method="post">
                <?php
                settings_fields(GH__SETTINGS_PAGE__ID);

                do_settings_sections(GH__SETTINGS_PAGE__ID);

                submit_button();
                ?>
                <small><?php esc_html_e('Saving will also clear the cache from WP Fastest Cache', GH__TEXT_DOMAIN); ?></small>
            </form>
            <div class="gh__backend__example-csp-table-container">
                <h4><?php _e('Example CSPs', GH__TEXT_DOMAIN); ?></h4>
                <table>
                    <tr>
                        <td>
                            <?php
                            _e('Default<br /><small>Blocks everything that is not from this website.</small>', GH__TEXT_DOMAIN);
                            ?>
                        </td>
                        <td>
                            <code>default-src 'self' data:</code>
                        </td>
                    </tr>
                    <tr>
                        <td>
                            <?php
                            _e('Default, but less strict<br /><small>May help with blocked scripts/styles/fonts.</small>', GH__TEXT_DOMAIN);
                            ?>
                        </td>
                        <td>
                            <code>default-src 'self' 'unsafe-inline' data:</code>
                        </td>
                    </tr>
                    <tr>
                        <td>
                            <?php
                            _e('OpenStreetMap<br /><small>Allows OpenStreetMap to be loaded.</small>', GH__TEXT_DOMAIN);
                            ?>
                        </td>
                        <td>
                            <code>
                                default-src 'self' 'unsafe-inline';<br />
                                connect-src 'self';<br />
                                frame-src 'none';<br />
                                img-src 'self' https://openlayers.org https://tile.openstreetmap.org http://a.tile.openstreetmap.org http://b.tile.openstreetmap.org http://c.tile.openstreetmap.org;<br />
                                script-src 'self' 'unsafe-eval' 'unsafe-inline' https://openlayers.org;<br />
                                style-src 'self' 'unsafe-inline' https://openlayers.org;
                            </code>
                        </td>
                    </tr>
                    <tr>
                        <td>
                            <?php
                            _e('Google Maps<br /><small>Allows Google Maps to be loaded.</small>', GH__TEXT_DOMAIN);
                            ?>
                        </td>
                        <td>
                            <code>
                                default-src 'self' 'unsafe-inline';<br />
                                script-src 'self' maps.googleapis.com;<br />
                                img-src 'self' data: maps.gstatic.com *.googleapis.com *.ggpht.com;
                            </code>
                        </td>
                    </tr>
                    <tr>
                        <td>
                            <?php
                            _e('Google Recaptcha<br /><small>Allows Google Recaptcha to be loaded.</small>', GH__TEXT_DOMAIN);
                            ?>
                        </td>
                        <td>
                            <code>
                                default-src 'self' 'unsafe-inline';<br />
                                script-src 'self' https://www.google.com/recaptcha/, https://www.gstatic.com/recaptcha/;<br />
                                frame-src 'self' https://www.google.com/recaptcha/, https://recaptcha.google.com/recaptcha/;
                            </code>
                        </td>
                    </tr>
                    <tr>
                        <td>
                            <?php
                            _e('Google Maps and Recaptcha<br /><small>Allows Google Recaptcha to be loaded.</small>', GH__TEXT_DOMAIN);
                            ?>
                        </td>
                        <td>
                            <code>
                                default-src 'self' 'unsafe-inline';<br />
                                script-src 'self' maps.googleapis.com https://www.google.com/recaptcha/, https://www.gstatic.com/recaptcha/;<br />
                                frame-src 'self' https://www.google.com/recaptcha/, https://recaptcha.google.com/recaptcha/;<br />
                                img-src 'self' data: maps.gstatic.com *.googleapis.com *.ggpht.com;
                            </code>
                        </td>
                    </tr>
                </table>
                <br />
                <?php
                _e('Please note that loading Google Maps or Google Recaptcha <strong>will</strong> lead to them loading Google Fonts. Therefore please also gather consent for Google Fonts and include the neccessary text in your privacy policy!', GH__TEXT_DOMAIN);
                ?>
            </div>
        </div>
    <?php
        }
    );
}

/* ########################################### */
/* ### renderer ### */
/* ########################################### */

function gdprhelper__renderer__checkbox_input($args)
{
    $field_name = esc_html($args[0]);
    $label_text = wp_kses_post($args[1]);
    $options = get_option(GH__SETTINGS_PAGE__ID);
    $value = isset($options[$field_name]) ? $options[$field_name] : '';
    $name = GH__SETTINGS_PAGE__ID . '[' . $field_name . ']';

    ?>
    <input type='checkbox' id='id__<?php echo $field_name; ?>' name='<?php echo $name; ?>' value='1' <?php checked(1, $value, true); ?> />
    <br />
    <label for='id__<?php echo $field_name; ?>'><?php echo $label_text; ?></label>
<?php
}

function gdprhelper__renderer__text_input($args)
{
    $field_name = esc_html($args[0]);
    $label_text = wp_kses_post($args[1]);
    $options = get_option(GH__SETTINGS_PAGE__ID);
    $value = isset($options[$field_name]) ? $options[$field_name] : '';
    $name = GH__SETTINGS_PAGE__ID . '[' . $field_name . ']';

?>
    <input type='text' id='id__<?php echo $field_name; ?>' name='<?php echo $name ?>' value='<?php echo $value; ?>' />
    <br />
    <label for='id__<?php echo $field_name; ?>'><?php echo $label_text; ?></label>
<?php
}

function gdprhelper__renderer__textarea_input($args)
{
    $field_name = esc_html($args[0]);
    $label_text = wp_kses_post($args[1]);
    $options = get_option(GH__SETTINGS_PAGE__ID);
    $value = isset($options[$field_name]) ? esc_textarea($options[$field_name]) : '';
    $name = GH__SETTINGS_PAGE__ID . '[' . $field_name . ']';

?>
    <textarea id='id__<?php echo $field_name; ?>' name='<?php echo $name ?>' rows='10' cols='100'><?php echo $value; ?></textarea>
    <br />
    <label for='id__<?php echo $field_name; ?>'><?php echo $label_text; ?></label>
<?php
}

function gdprhelper__renderer__image_input($args)
{
    $field_name = esc_html($args[0]);
    $label_text = wp_kses_post($args[1]);
    $options = get_option(GH__SETTINGS_PAGE__ID);
    $value = isset($options[$field_name]) ? esc_textarea($options[$field_name]) : '';
    $name = GH__SETTINGS_PAGE__ID . '[' . $field_name . ']';
    $id = str_replace('[', '-', $name);
    $id = str_replace(']', '-', $id);

    $image_thumb_url = '';
    if ($value) {
        $image_thumb_url = wp_get_attachment_thumb_url($value);
    }

?>

    <input id="id__<?php echo $id; ?>" class="gh__backend__uploader_value" type="hidden" name="<?php echo $name; ?>" value="<?php echo $value; ?>" />
    <img id="id__<?php echo $id; ?>_img_preview" class="gh__backend__uploader_preview" src="<?php echo ($image_thumb_url); ?>" />

    <br />

    <input id="id__<?php echo $id; ?>_btn_upload" type="button" data-gh-uploader-title="<?php _e('Upload an image as map placeholder', GH__TEXT_DOMAIN); ?>" data-gh-uploader-accept-text="<?php _e('Use image as map placeholder', GH__TEXT_DOMAIN); ?>" class="button gh__backend__uploader_btn_select_image" value="<?php _e('Upload new image', GH__TEXT_DOMAIN); ?>" />

    <input id="id__<?php echo $id; ?>_btn_delete" type="button" class="button gh__backend__uploader_btn_delete" value="<?php _e('Remove image', GH__TEXT_DOMAIN); ?>" />


    <br />

    <label><?php echo $label_text; ?></label>

<?php
}
