<?php

defined('ABSPATH') || die('error');

/**
 * Entry point for the whole plugin. Sets the Content-Security-Policy (CSP) http header. If is_admin then the backend settings page will also be build
 */
function gdprhelper__main($pluginfile): void
{
    require_once(GH__PLUGIN_INCLUDES_PATH . '/includes/magicStrings.php');
    require_once(GH__PLUGIN_INCLUDES_PATH . '/includes/elementor/helper.php');

    add_action('init', 'gdprhelper__load_language_domain');

    add_action('init', 'gdprhelper__set_cookies');

    $is_backend = is_admin() && !wp_doing_ajax();

    if ($is_backend) {
        require_once(GH__PLUGIN_INCLUDES_PATH . '/includes/backendPage.php');
        gdprhelper__main_backend($pluginfile);
    } else {
        wp_enqueue_script('gh__frontend_js', GH__PLUGIN_URL . '/js/frontend_script.js', array('jquery'), GH__CURRENT_VERSION, true);
        wp_enqueue_style('gh__frontend_css', GH__PLUGIN_URL . '/css/frontend_style.css', [], GH__CURRENT_VERSION);
    }

    add_action('init', 'gdprhelper__load_in_init');

    // Lastly set the correct CSP header
    add_action('init', 'gdprhelper__setHeader');
}

function gdprhelper__load_in_init()
{
    if (gdprhelper__helper__is_elementor_installed_and_active()) {
        // Load map placeholder if enabled
        $option = get_option(GH__SETTINGS_PAGE__ID);
        if (gdprhelper__option_equals_value($option, GH__SETTINGS_FIELDS__MAPS_HELPER_ENABLED, true)) {
            require_once(GH__PLUGIN_INCLUDES_PATH . '/includes/elementor/mapHandler.php');
            gdprhelper__map_handler__entry();
        }

        // Add custom Elementor widgets
        require_once(GH__PLUGIN_INCLUDES_PATH . '/includes/elementor/widgetShortcode.php');
        gdprhelper__custom_elementor__entry();
    }
}

function gdprhelper__set_cookies()
{
    if (isset($_GET[GH__COOKIES__PRIVACY_POLICY_ACCEPTED]) && ($_GET[GH__COOKIES__PRIVACY_POLICY_ACCEPTED] == 'true')) {
        $id = (int) $_GET['id'];
        check_admin_referer(GH__COOKIES__PRIVACY_POLICY_ACCEPTED . $id);

        setcookie(GH__COOKIES__PRIVACY_POLICY_ACCEPTED, 'yes', 0, COOKIEPATH, COOKIE_DOMAIN); // Session cookie

        $url = $_SERVER['REQUEST_URI'];
        $scrollto = $_REQUEST['scrollto'];
        $url = remove_query_arg(array(GH__COOKIES__PRIVACY_POLICY_ACCEPTED, 'scrollto', 'id', '_wpnonce'), $url);
        $url = add_query_arg('clearcache', $id . '#' . $scrollto, $url);
        wp_redirect($url);

        die();
    }
}

/**
 * Sets the CSP http header
 */
function gdprhelper__setHeader(): void
{
    $option = get_option(GH__SETTINGS_PAGE__ID);

    if (gdprhelper__option_equals_value($option, GH__SETTINGS_FIELDS__ENABLED, false)) {
        return;
    }

    if (is_user_logged_in() && !gdprhelper__option_equals_value($option, GH__SETTINGS_FIELDS__ENABLED_WHEN_LOGGED_IN, true)) {
        return;
    }

    $header = 'Content-Security-Policy';
    if (gdprhelper__option_equals_value($option, GH__SETTINGS_FIELDS__REPORT_ONLY, true)) {
        $header .= '-Report-Only';
    }

    $csp_string = $option[GH__SETTINGS_FIELDS__CSP_STRING];
    $csp_string_without_newlines = preg_replace('/\R/u', ' ', $csp_string);
    $csp_content_parsed = gdprhelper__parse_csp_through_additional_settings($csp_string_without_newlines);
    header(sprintf('%s: %s', $header, $csp_content_parsed));

    if (isset($option[GH__SETTINGS_FIELDS__REPORT_ONLY]) && !empty($option[GH__SETTINGS_FIELDS__REPORT_URI])) {
        header(sprintf('Report-To: %s', $option['general']['report_to_uri']));
    }
}

function gdprhelper__parse_csp_through_additional_settings($csp)
{
    // var_dump("csp starting");
    // var_dump($csp);

    $maps_enabled = false;

    // Checks
    if ($_COOKIE[GH__COOKIES__PRIVACY_POLICY_ACCEPTED]) {
        $maps_enabled = true;
    }

    if (!$maps_enabled) {
        return $csp;
    }

    // Parse CSP

    $new_csp = gdprhelper__build_new_csp($csp, $maps_enabled); // first run will add missing directives as empty directives
    // var_dump("<br><br><hr><br><br>second run<br>");
    $new_csp = gdprhelper__build_new_csp($new_csp, $maps_enabled); // seconds run will fill those newly added directives

    // var_dump("<br><br><hr><br>");
    // var_dump($new_csp);

    $new_csp = str_replace('; ;', ';', $new_csp);
    $new_csp = str_replace('  ', ' ', $new_csp);

    // var_dump("<br><br><hr><br>");
    // var_dump($new_csp);

    return $new_csp;
}

function gdprhelper__build_new_csp($csp, $maps_enabled)
{
    $directives = explode(';', $csp);

    $arr_available_directive_names = array();
    foreach ($directives as $directive) {
        $splitter = explode(' ', trim($directive));
        $directive_name = trim($splitter[0]);

        array_push($arr_available_directive_names, $directive_name);
    }

    $new_csp = '';
    foreach ($directives as $directive) {
        $splitter = explode(' ', trim($directive));
        $directive_name = trim($splitter[0]);

        // var_dump("<br><br>directive before");
        // var_dump($directive);

        if ($maps_enabled) {
            // General
            gdprhelper__add_csp_directive_value_if_not_present($directive, $arr_available_directive_names, $directive_name, 'default-src', '\'self\'');
            gdprhelper__add_csp_directive_value_if_not_present($directive, $arr_available_directive_names, $directive_name, 'default-src', '\'unsafe-inline\'');

            gdprhelper__add_csp_directive_value_if_not_present($directive, $arr_available_directive_names, $directive_name, 'style-src', '\'self\'');

            gdprhelper__add_csp_directive_value_if_not_present($directive, $arr_available_directive_names, $directive_name, 'connect-src', '\'self\'');

            gdprhelper__add_csp_directive_value_if_not_present($directive, $arr_available_directive_names, $directive_name, 'script-src', '\'self\'');

            // Google Maps
            gdprhelper__add_csp_directive_value_if_not_present($directive, $arr_available_directive_names, $directive_name, 'script-src', 'maps.googleapis.com');

            gdprhelper__add_csp_directive_value_if_not_present($directive, $arr_available_directive_names, $directive_name, 'frame-src', 'https://maps.google.com/');
            gdprhelper__add_csp_directive_value_if_not_present($directive, $arr_available_directive_names, $directive_name, 'frame-src', 'https://www.google.com/');

            gdprhelper__add_csp_directive_value_if_not_present($directive, $arr_available_directive_names, $directive_name, 'img-src', '\'self\'');
            gdprhelper__add_csp_directive_value_if_not_present($directive, $arr_available_directive_names, $directive_name, 'img-src', 'data:');
            gdprhelper__add_csp_directive_value_if_not_present($directive, $arr_available_directive_names, $directive_name, 'img-src', 'maps.gstatic.com');
            gdprhelper__add_csp_directive_value_if_not_present($directive, $arr_available_directive_names, $directive_name, 'img-src', '*.googleapis.com');
            gdprhelper__add_csp_directive_value_if_not_present($directive, $arr_available_directive_names, $directive_name, 'img-src', '*.ggpht.com');

            // Open Street Map
            gdprhelper__add_csp_directive_value_if_not_present($directive, $arr_available_directive_names, $directive_name, 'img-src', 'https://openlayers.org');
            gdprhelper__add_csp_directive_value_if_not_present($directive, $arr_available_directive_names, $directive_name, 'img-src', 'https://tile.openstreetmap.org');
            gdprhelper__add_csp_directive_value_if_not_present($directive, $arr_available_directive_names, $directive_name, 'img-src', 'http://a.tile.openstreetmap.org');
            gdprhelper__add_csp_directive_value_if_not_present($directive, $arr_available_directive_names, $directive_name, 'img-src', 'http://b.tile.openstreetmap.org');
            gdprhelper__add_csp_directive_value_if_not_present($directive, $arr_available_directive_names, $directive_name, 'img-src', 'http://c.tile.openstreetmap.org');

            gdprhelper__add_csp_directive_value_if_not_present($directive, $arr_available_directive_names, $directive_name, 'script-src', '\'unsafe-eval\'');
            gdprhelper__add_csp_directive_value_if_not_present($directive, $arr_available_directive_names, $directive_name, 'script-src', '\'unsafe-inline\'');

            gdprhelper__add_csp_directive_value_if_not_present($directive, $arr_available_directive_names, $directive_name, 'style-src', '\'unsafe-inline\'');
            gdprhelper__add_csp_directive_value_if_not_present($directive, $arr_available_directive_names, $directive_name, 'style-src', 'https://openlayers.org');
        }

        // var_dump("<br><br>directive after");
        // var_dump($directive);

        $new_csp .= $directive . '; ';
    }

    return $new_csp;
}

function gdprhelper__add_csp_directive_value_if_not_present(&$directive, &$arr_available_directive_names, $directive_name, $target_directive_name, $value)
{
    // var_dump("comparing directive name '$directive_name' with target of '$target_directive_name'<br>");
    if ($directive_name !== $target_directive_name) {
        // var_dump("does <strong>not equal</strong><br>");
        if (!in_array($target_directive_name, $arr_available_directive_names, true)) {
            // var_dump("added new directive '$target_directive_name'<br>");
            // var_dump($arr_available_directive_names);
            $directive .= '; ' . $target_directive_name . ' ';
            array_push($arr_available_directive_names, $target_directive_name);
        } else {
            return;
        }
    }

    if (!str_contains($directive, $value)) {
        $directive .= ' ' . $value;
    }
}

function gdprhelper__option_equals_value($options, $key, $value): bool
{
    return isset($options[$key]) && $options[$key] == $value;
}

function gdprhelper__load_language_domain(): void
{
    load_plugin_textdomain(GH__TEXT_DOMAIN, false, dirname(plugin_basename(__FILE__)) . '/languages');
}
